@@ -1,16 +1,16 @@
 /*
- * Copyright (c) 2014-2017, Nils Christopher Brause
+ * Copyright (c) 2014-2019, Nils Christopher Brause, Philipp Kerling
  * All rights reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
- * 
+ *
  * 1. Redistributions of source code must retain the above copyright notice, this
- *    list of conditions and the following disclaimer. 
+ *    list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -43,7 +43,7 @@ namespace wayland
    *
    * Log message is the first argument
    */
-  typedef std::function<void(std::string)> log_handler;
+  using log_handler = std::function<void(std::string)> ;
 
   /** \brief Set C library log handler
    *
@@ -55,8 +55,8 @@ namespace wayland
    */
   void set_log_handler(log_handler handler);
 
-  /** \brief A queue for proxy_t object events. 
-    
+  /** \brief A queue for proxy_t object events.
+
       Event queues allows the events on a display to be handled in a
       thread-safe manner. See display_t for details.
   */
@@ -65,7 +65,12 @@ namespace wayland
     event_queue_t(wl_event_queue *q);
     friend class display_t;
   public:
-    event_queue_t();
+    event_queue_t() = default;
+    event_queue_t(const event_queue_t&) = default;
+    event_queue_t(event_queue_t&&) noexcept = default;
+    event_queue_t& operator=(const event_queue_t&) = default;
+    event_queue_t& operator=(event_queue_t&&) noexcept = default;
+    ~event_queue_t() noexcept = default;
   };
 
   class display_t;
@@ -76,7 +81,12 @@ namespace wayland
     // base class for event listener storage.
     struct events_base_t
     {
-      virtual ~events_base_t() { }
+      events_base_t() = default;
+      events_base_t(const events_base_t&) = default;
+      events_base_t(events_base_t&&) noexcept = default;
+      events_base_t& operator=(const events_base_t&) = default;
+      events_base_t& operator=(events_base_t&&) noexcept = default;
+      virtual ~events_base_t() noexcept = default;
     };
   }
 
@@ -141,6 +151,12 @@ namespace wayland
     friend class detail::argument_t;
     friend struct detail::proxy_data_t;
 
+    // Interface desctiption filled in by the each interface class
+    const wl_interface *interface = nullptr;
+
+    // copy constructor filled in by the each interface class
+    std::function<proxy_t(proxy_t)> copy_constructor;
+
     // universal dispatcher
     static int c_dispatcher(const void *implementation, void *target,
                             uint32_t opcode, const wl_message *message,
@@ -148,14 +164,11 @@ namespace wayland
 
     // marshal request
     proxy_t marshal_single(uint32_t opcode, const wl_interface *interface,
-                           std::vector<detail::argument_t> v, std::uint32_t version = 0);
+                           const std::vector<detail::argument_t>& args, std::uint32_t version = 0);
 
   protected:
-    // Interface desctiption filled in by the each interface class
-    const wl_interface *interface = nullptr;
-
-    // copy constructor filled in by the each interface class
-    std::function<proxy_t(proxy_t)> copy_constructor;
+    void set_interface(const wl_interface *iface);
+    void set_copy_constructor(const std::function<proxy_t(proxy_t)>& func);
 
     friend class registry_t;
     // marshal a request, that doesn't lead a new proxy
@@ -166,16 +179,16 @@ namespace wayland
     // - std::string
     // - array_t
     template <typename...T>
-    void marshal(uint32_t opcode, T...args)
+    void marshal(uint32_t opcode, const T& ...args)
     {
       std::vector<detail::argument_t> v = { detail::argument_t(args)... };
-      marshal_single(opcode, NULL, v);
+      marshal_single(opcode, nullptr, v);
     }
 
     // marshal a request that leads to a new proxy with inherited version
     template <typename...T>
     proxy_t marshal_constructor(uint32_t opcode, const wl_interface *interface,
-                                T...args)
+                                const T& ...args)
     {
       std::vector<detail::argument_t> v = { detail::argument_t(args)... };
       return marshal_single(opcode, interface, v);
@@ -184,7 +197,7 @@ namespace wayland
     // marshal a request that leads to a new proxy with specific version
     template <typename...T>
     proxy_t marshal_constructor_versioned(uint32_t opcode, const wl_interface *interface,
-                                          uint32_t version, T...args)
+                                          uint32_t version, const T& ...args)
     {
       std::vector<detail::argument_t> v = { detail::argument_t(args)... };
       return marshal_single(opcode, interface, v, version);
@@ -199,17 +212,17 @@ namespace wayland
       new. Will automatically be deleted upon destruction.
     */
     void set_events(std::shared_ptr<detail::events_base_t> events,
-                    int(*dispatcher)(uint32_t, std::vector<detail::any>, std::shared_ptr<detail::events_base_t>));
+                    int(*dispatcher)(uint32_t, const std::vector<detail::any>&, const std::shared_ptr<detail::events_base_t>&));
 
     // Retrieve the previously set user data
     std::shared_ptr<detail::events_base_t> get_events();
 
     // Constructs NULL proxies.
-    proxy_t();
-    
+    proxy_t() = default;
+
     struct construct_proxy_wrapper_tag {};
     // Construct from proxy as wrapper
-    proxy_t(const proxy_t &wrapped_proxy, construct_proxy_wrapper_tag);
+    proxy_t(const proxy_t &wrapped_proxy, construct_proxy_wrapper_tag /*unused*/);
 
   public:
     /** \brief Cronstruct a proxy_t from a wl_proxy pointer
@@ -239,14 +252,14 @@ namespace wayland
         Transfers the contents of the proxy_t object on the right
         to the proxy_t object being constructed.
     */
-    proxy_t(proxy_t &&p);
+    proxy_t(proxy_t &&p) noexcept;
 
     /** \brief Move Asignment operator
         \param p A proxy_t object
 
         Swaps the contents of both proxy_t objects.
     */
-    proxy_t &operator=(proxy_t &&p);
+    proxy_t &operator=(proxy_t &&p) noexcept;
 
     /** \brief Destructor
 
@@ -264,21 +277,21 @@ namespace wayland
     */
     uint32_t get_id() const;
 
-    /** \brief Get the interface name (class) of a proxy object. 
-        \return The interface name of the object associated with the proxy 
+    /** \brief Get the interface name (class) of a proxy object.
+        \return The interface name of the object associated with the proxy
     */
     std::string get_class() const;
-    
+
     /** \brief Get the protocol object version of a proxy object.
-     * 
+     *
      * Gets the protocol object version of a proxy object, or 0 if the proxy was
      * created with unversioned API.
-     * 
+     *
      * A returned value of 0 means that no version information is available,
      * so the caller must make safe assumptions about the object's real version.
-     * 
+     *
      * \ref display_t will always return version 0.
-     * 
+     *
      * \return The protocol object version of the proxy or 0
      */
     uint32_t get_version() const;
@@ -291,7 +304,7 @@ namespace wayland
     }
 
     /** \brief Assign a proxy to an event queue.
-        \param queue The event queue that will handle this proxy 
+        \param queue The event queue that will handle this proxy
 
         Assign proxy to event queue. Events coming from proxy will be queued in
         queue instead of the display's main queue.
@@ -299,35 +312,35 @@ namespace wayland
         See also: display_t::dispatch_queue().
     */
     void set_queue(event_queue_t queue);
-    
+
     /** \brief Get a pointer to the underlying C struct.
      *  \return The underlying wl_proxy wrapped by this proxy_t if it exists,
      *          otherwise an exception is thrown
      */
     wl_proxy *c_ptr() const;
-    
+
     /** \brief Check whether this wrapper actually wraps an object
      *  \return true if there is an underlying object, false if this wrapper is
      *          empty
      */
     bool proxy_has_object() const;
-    
+
     /** \brief Check whether this wrapper actually wraps an object
      *  \return true if there is an underlying object, false if this wrapper is
      *          empty
      */
     operator bool() const;
-    
+
     /** \brief Check whether two wrappers refer to the same object
      */
     bool operator==(const proxy_t &right) const;
 
     /** \brief Check whether two wrappers refer to different objects
-     */    
+     */
     bool operator!=(const proxy_t &right) const;
-    
+
     /** \brief Release the wrapped object (if any), making this an empty wrapper
-     * 
+     *
      * Note that display_t instances cannot be released this way. Attempts to
      * do so are ignored.
      */
@@ -336,67 +349,68 @@ namespace wayland
 
   /** \brief Represents an intention to read from the display file
    *  descriptor
-   * 
+   *
    * When not using the convenience method \ref display_t::dispatch that takes care
    * of this automatically, threads that want to read events from a Wayland
    * display file descriptor must announce their intention to do so beforehand - in the C API,
    * this is done using wl_display_prepare_read. This intention must then be
    * resolved either by actually invoking a read from the file descriptor or cancelling.
-   * 
+   *
    * This RAII class makes sure that when it goes out of scope, the intent
    * is cancelled automatically if it was not finalized by manually cancelling
    * or reading before. Otherwise, it would be easy to forget resolving the
    * intent e.g. when handling errors, potentially leading to a deadlock.
-   * 
+   *
    * Read intents can only be created by a \ref display_t with
    * \ref display_t::obtain_read_intent and \ref display_t::obtain_queue_read_intent.
-   * 
+   *
    * Undefined behavior occurs when the associated \ref display_t or
    * \ref event_queue_t is destroyed when a read_intent has not been finalized yet.
    */
   class read_intent
   {
   public:
-    read_intent(read_intent &&other) = default;
+    read_intent(read_intent &&other) noexcept = default;
+    read_intent(read_intent const &other) = delete;
+    read_intent& operator=(read_intent const &other) = delete;
+    read_intent& operator=(read_intent &&other) noexcept = delete;
     ~read_intent();
-    
+
     /** \brief Check whether this intent was already finalized with \ref cancel
      * or \ref read
      */
     bool is_finalized() const;
 
     /** \brief Cancel read intent
-     * 
+     *
      * An exception is thrown when the read intent was already finalized.
      */
     void cancel();
 
     /** \brief Read events from display file descriptor
-     * 
+     *
      * This will read events from the file descriptor for the
      * display. This function does not dispatch events, it only reads
      * and queues events into their corresponding event queues. If no
      * data is avilable on the file descriptor, read() returns immediately.
      * To dispatch events that may have been queued, call
      * \ref display_t::dispatch_pending or \ref display_t::dispatch_queue_pending.
-     * 
+     *
      * An exception is thrown when the read intent was already finalized.
      * Each read intent can only be used for reading once. A new one must be
      * obtained for any further read requests.
      */
     void read();
-    
+
   private:
     read_intent(wl_display *display, wl_event_queue *event_queue = nullptr);
     friend class display_t;
-    read_intent(read_intent const &other) = delete;
-    read_intent& operator=(read_intent const &other) = delete;
-    
+
     wl_display *display;
     wl_event_queue *event_queue = nullptr;
     bool finalized = false;
   };
-  
+
   class callback_t;
   class registry_t;
 
@@ -463,24 +477,25 @@ namespace wayland
   class display_t : public proxy_t
   {
   private:
-    display_t(const display_t &d) { }
     // Construct as proxy wrapper
-    display_t(proxy_t const &wrapped_proxy, construct_proxy_wrapper_tag);
+    display_t(proxy_t const &wrapped_proxy, construct_proxy_wrapper_tag /*unused*/);
 
   public:
     /** \brief Connect to Wayland display on an already open fd.
         \param fd The fd to use for the connection
-      
+
         The display_t takes ownership of the fd and will close it when
         the display is destroyed. The fd will also be closed in case of
         failure.
     */
     display_t(int fd);
 
-    display_t(display_t &&d);
-    display_t &operator=(display_t &&d);
+    display_t(display_t &&d) noexcept;
+    display_t(const display_t &d) = delete;
+    display_t &operator=(const display_t &d) = delete;
+    display_t &operator=(display_t &&d) noexcept;
 
-    /**  \brief Connect to a Wayland display. 
+    /**  \brief Connect to a Wayland display.
          \param name Optional name of the Wayland display to connect to
 
          Connect to the Wayland display named name. If name is empty,
@@ -488,19 +503,45 @@ namespace wayland
          variable if it is set, otherwise display "wayland-0" will be
          used.
     */
-    display_t(std::string name = "");
+    display_t(const std::string& name = {});
+
+    /** \brief Use an existing connection to a Wayland display to
+               construct a waylandpp display_t
+        \param display C wl_display pointer to use; must not be nullptr
+
+        A wl_display* that was already established using the C wayland-client
+        API is wrapped in an waylandpp display_t instance so it can be used
+        easily from C++. Ownership of the display is not taken, so this may
+        be used for wrapping a wl_display connection established by another library.
+
+        On destruction of the display_t, wl_display_disconnect is not called and
+        no resources are freed.
+        It is the responsibility of the caller to make sure that the wl_display
+        and the display_t are not used simultaneously in incompatible ways. It is
+        especially problematic if the wl_display is destroyed while the display_t
+        wrapper is still being used.
+
+        Whether the wl_display or the display_t is destructed first ultimately
+        does not matter, but any waylandpp proxy_t instances must be destructed
+        or have their owned objects released before the wl_display is destroyed.
+        Otherwise, the proxy_t destructor will try to free the underlying wl_proxy
+        that was already destroyed together with the wl_display.
+     */
+    explicit display_t(wl_display* display);
 
     /** \brief Close a connection to a Wayland display.
 
         Close the connection to display and free all resources
         associated with it.
+        This does not apply to display_t instances that are wrappers for
+        a pre-established C wl_display.
     */
-    ~display_t();
+    ~display_t() noexcept = default;
 
     /** \brief Create a new event queue for this display.
         \return A new event queue associated with this display or NULL
         on failure.
-    */ 
+    */
     event_queue_t create_queue();
 
     /** \brief Get a display context's file descriptor.
@@ -509,7 +550,7 @@ namespace wayland
         Return the file descriptor associated with a display so it can
         be integrated into the client's main loop.
     */
-    int get_fd();
+    int get_fd() const;
 
     /** \brief Block until all pending request are processed by the server.
         \return The number of dispatched events
@@ -532,21 +573,21 @@ namespace wayland
          that calling roundtrip_queue() doesn't interfere with calling
          prepare_read() and read_events())
     */
-    int roundtrip_queue(event_queue_t queue);
-    
+    int roundtrip_queue(const event_queue_t& queue);
+
     /** \brief Announce calling thread's intention to read events from the
      * Wayland display file descriptor
-     * 
+     *
      * This ensures that until the thread is ready to read and calls \ref
      * read_intent::read, no other thread will read from the file descriptor.
      * During preparation, all undispatched events in the event queue
      * are dispatched until the queue is empty.
-     * 
+     *
      * Use this function before polling on the display fd or to integrate the
      * fd into a toolkit event loop in a race-free way.
-     * 
+     *
      * Typical usage is:
-     * 
+     *
      * \code
      * auto read_intent = display.obtain_read_intent();
      * display.flush();
@@ -555,31 +596,31 @@ namespace wayland
      *   read_intent.read();
      * display.dispatch_pending();
      * \endcode
-     * 
+     *
      * The \ref read_intent ensures that if the above code e.g. throws an
      * exception before actually reading from the file descriptor or times out
      * in poll(), the read intent is always cancelled so other threads can proceed.
-     * 
+     *
      * In one thread, do not hold more than one read intent for the same
      * display at the same time, irrespective of the event queue.
-     * 
+     *
      * \return New \ref read_intent for this display and the default event queue
      * \exception std::system_error on failure
      */
     read_intent obtain_read_intent();
-    
+
     /** \brief Announce calling thread's intention to read events from the
      * Wayland display file descriptor
-     * 
+     *
      * \param queue event queue for which the read event will be valid
      * \return New \ref read_intent for this display and the specified event queue
      * \exception std::system_error on failure
-     * 
+     *
      * See \ref obtain_read_intent for details.
      */
-    read_intent obtain_queue_read_intent(event_queue_t queue);
+    read_intent obtain_queue_read_intent(const event_queue_t& queue);
 
-    /** \brief Dispatch events in an event queue. 
+    /** \brief Dispatch events in an event queue.
         \param queue The event queue to dispatch
         \return The number of dispatched events
         \exception std::system_error on failure
@@ -592,11 +633,11 @@ namespace wayland
         calling from the main thread, it will block reading data from
         the display fd. For other threads this will block until the main
         thread queues events on the queue passed as argument.
-    */ 
-    int dispatch_queue(event_queue_t queue);
+    */
+    int dispatch_queue(const event_queue_t& queue);
 
     /** \brief Dispatch pending events in an event queue.
-        \param queue The event queue to dispatch 
+        \param queue The event queue to dispatch
         \return The number of dispatched events
         \exception std::system_error on failure
 
@@ -605,14 +646,14 @@ namespace wayland
         appropriately. If there are no events queued, this function
         returns immediately.
     */
-    int dispatch_queue_pending(event_queue_t queue);
-      
-    /** \brief Process incoming events. 
+    int dispatch_queue_pending(const event_queue_t& queue);
+
+    /** \brief Process incoming events.
         \return The number of dispatched events
         \exception std::system_error on failure
 
         Dispatch the display's main event queue.
-      
+
         If the main event queue is empty, this function blocks until
         there are events to be read from the display fd. Events are read
         and queued on the appropriate event queues. Finally, events on
@@ -632,11 +673,11 @@ namespace wayland
     /** \brief Dispatch main queue events without reading from the display fd.
         \return The number of dispatched events
         \exception std::system_error on failure
-      
+
         This function dispatches events on the main event queue. It
         does not attempt to read the display fd and simply returns zero
         if the main queue is empty, i.e., it doesn't block.
-       
+
         This is necessary when a client's main loop wakes up on some fd
         other than the display fd (network socket, timer fd, etc) and
         calls wl_display_dispatch_queue() from that callback. This may
@@ -645,14 +686,14 @@ namespace wayland
         to block, the display fd no longer has data, causing a call to
         poll(2) (or similar functions) to block indefinitely, even
         though there are events ready to dispatch.
-       
+
         To proper integrate the wayland display fd into a main loop,
         the client should always call display_t::dispatch_pending() and
         then display_t::flush() prior to going back to sleep. At that
         point, the fd typically doesn't have data so attempting I/O
         could block, but events queued up on the main queue should be
         dispatched.
-       
+
         A real-world example is a main loop that wakes up on a timerfd
         (or a sound card fd becoming writable, for example in a video
         player), which then triggers GL rendering and eventually
@@ -679,11 +720,11 @@ namespace wayland
     */
     int get_error() const;
 
-    /** \brief Send all buffered requests on the display to the server. 
+    /** \brief Send all buffered requests on the display to the server.
         \return Tuple of the number of bytes sent and whether all data
                 was sent.
         \exception std::system_error on failure
-      
+
         Send all buffered data on the client side to the server. Clients
         should call this function before blocking. On success, the
         number of bytes sent to the server is returned.
@@ -696,7 +737,7 @@ namespace wayland
     std::tuple<int, bool> flush();
 
     /** \brief asynchronous roundtrip
-      
+
         The sync request asks the server to emit the 'done' event on
         the returned callback_t object. Since requests are handled
         in-order and events are delivered in-order, this can be used as
@@ -715,7 +756,7 @@ namespace wayland
         list and bind the global objects available from the compositor.
     */
     registry_t get_registry();
-    
+
     operator wl_display*() const;
 
     /** \brief create proxy wrapper for this display
