@@ -38,7 +38,7 @@ struct element_t
   std::string description;
   static const std::set<std::string> keywords;
 
-  std::string sanitise(std::string str)
+  static std::string sanitise(std::string str)
   {
     if(keywords.count(str))
       return "_" + str;
@@ -52,106 +52,103 @@ struct argument_t : public element_t
   std::string interface;
   std::string enum_iface;
   std::string enum_name;
-  bool allow_null;
+  bool allow_null = false;
 
-  std::string print_enum_wire_type()
+  std::string print_enum_wire_type() const
   {
     // Enums can be int or uint on the wire, except for bitfields
     if(type == "int")
       return "int32_t";
-    else if(type == "uint")
+    if(type == "uint")
       return "uint32_t";
-    else
-      throw std::runtime_error("Enum type must be int or uint");
+    throw std::runtime_error("Enum type must be int or uint");
   }
 
-  std::string print_type()
+  std::string print_type() const
   {
-    if(interface != "")
+    if(!interface.empty())
       return interface + "_t";
-    else if(enum_iface != "")
+    if(!enum_iface.empty())
       return enum_iface + "_" + enum_name;
-    else if(type == "int")
+    if(type == "int")
       return "int32_t";
-    else if(type == "uint")
+    if(type == "uint")
       return "uint32_t";
-    else if(type == "fixed")
+    if(type == "fixed")
       return "double";
-    else if(type == "string")
+    if(type == "string")
       return "std::string";
-    else if(type == "object")
+    if(type == "object")
       return "proxy_t";
-    else if(type == "new_id")
+    if(type == "new_id")
       return "proxy_t";
-    else if(type == "fd")
+    if(type == "fd")
       return "int";
-    else if(type == "array")
+    if(type == "array")
       return "array_t";
-    else
-      return type;
+    return type;
   }
 
-  std::string print_short()
+  std::string print_short() const
   {
     if(type == "int")
       return "i";
-    else if(type == "uint")
+    if(type == "uint")
       return "u";
-    else if(type == "fixed")
+    if(type == "fixed")
       return "f";
-    else if(type == "string")
+    if(type == "string")
       return "s";
-    else if(type == "object")
+    if(type == "object")
       return "o";
-    else if(type == "new_id")
+    if(type == "new_id")
       return "n";
-    else if(type == "array")
+    if(type == "array")
       return "a";
-    else if(type == "fd")
+    if(type == "fd")
       return "h";
-    else
-      return "x";
+    return "x";
   }
 
-  std::string print_argument()
+  std::string print_argument() const
   {
-    return print_type() + " " + sanitise(name);
+    return print_type() + (!interface.empty() || !enum_iface.empty() || type == "string" || type == "array" ? " const& " : " ") + sanitise(name);
   }
 };
 
 struct event_t : public element_t
 {
   std::list<argument_t> args;
-  int since;
+  int since = 0;
 
-  std::string print_functional()
+  std::string print_functional() const
   {
     std::stringstream ss;
     ss << "    std::function<void(";
-    for(auto &arg : args)
+    for(auto const& arg : args)
       ss << arg.print_type() << ", ";
-    if(args.size())
+    if(!args.empty())
       ss.str(ss.str().substr(0, ss.str().size()-2));
     ss.seekp(0, std::ios_base::end);
     ss << ")> " << sanitise(name) << ";";
     return ss.str();
   }
 
-  std::string print_dispatcher(int opcode)
+  std::string print_dispatcher(int opcode) const
   {
     std::stringstream ss;
     ss << "    case " << opcode << ":" << std::endl
        << "      if(events->" << sanitise(name) << ") events->" << sanitise(name) << "(";
 
     int c = 0;
-    for(auto &arg : args)
-      if(arg.enum_name != "" && arg.type != "array")
+    for(auto const& arg : args)
+      if(!arg.enum_name.empty() && arg.type != "array")
         ss << arg.print_type() << "(args[" << c++ << "].get<" << arg.print_enum_wire_type() << ">()), ";
-      else if(arg.interface != "")
+      else if(!arg.interface.empty())
         ss << arg.print_type() << "(args[" << c++ << "].get<proxy_t>()), ";
       else
         ss << "args[" << c++ << "].get<" << arg.print_type() << ">(), ";
-    if(args.size())
+    if(!args.empty())
       ss.str(ss.str().substr(0, ss.str().size()-2));
     ss.seekp(0, std::ios_base::end);
     ss << ");" << std::endl
@@ -159,32 +156,32 @@ struct event_t : public element_t
     return ss.str();
   }
 
-  std::string print_signal_header()
+  std::string print_signal_header() const
   {
     std::stringstream ss;
     ss << "  /** \\brief " << summary << std::endl;
-    for(auto &arg : args)
+    for(auto const& arg : args)
       ss << "      \\param " << arg.name << " " << arg.summary << std::endl;
     ss << description << std::endl
        << "  */" << std::endl;
 
     ss << "  std::function<void(";
-    for(auto &arg : args)
+    for(auto const& arg : args)
       ss << arg.print_type() + ", ";
-    if(args.size())
+    if(!args.empty())
       ss.str(ss.str().substr(0, ss.str().size()-2));
     ss.seekp(0, std::ios_base::end);
     ss << ")> &on_" <<  name << "();" << std::endl;
     return ss.str();
   }
 
-  std::string print_signal_body(std::string interface_name)
+  std::string print_signal_body(const std::string& interface_name) const
   {
     std::stringstream ss;
     ss << "std::function<void(";
-    for(auto &arg : args)
+    for(auto const& arg : args)
       ss << arg.print_type() << ", ";
-    if(args.size())
+    if(!args.empty())
       ss.str(ss.str().substr(0, ss.str().size()-2));
     ss.seekp(0, std::ios_base::end);
     ss << ")> &" + interface_name + "_t::on_" + name + "()" << std::endl
@@ -198,32 +195,31 @@ struct event_t : public element_t
 struct request_t : public event_t
 {
   argument_t ret;
-  int opcode;
+  int opcode = 0;
 
-  std::string availability_function_name()
+  std::string availability_function_name() const
   {
     if(since > 1)
       return std::string("can_") + name;
-    else
-      return "";
+    return "";
   }
 
-  std::string since_version_constant_name()
+  std::string since_version_constant_name() const
   {
     return name + "_since_version";
   }
 
-  std::string print_header()
+  std::string print_header() const
   {
     std::stringstream ss;
     ss << "  /** \\brief " << summary << std::endl;
-    if(ret.summary != "")
+    if(!ret.summary.empty())
       ss << "      \\return " << ret.summary << std::endl;
-    for(auto &arg : args)
+    for(auto const& arg : args)
       {
         if(arg.type == "new_id")
           {
-            if(arg.interface == "")
+            if(arg.interface.empty())
               ss << "      \\param interface Interface to bind" << std::endl
                  << "      \\param version Interface version" << std::endl;
           }
@@ -233,16 +229,16 @@ struct request_t : public event_t
     ss << description << std::endl
        << "  */" << std::endl;
 
-    if(ret.name == "")
+    if(ret.name.empty())
       ss << "  void ";
     else
       ss << "  " << ret.print_type() << " ";
     ss << sanitise(name) << "(";
 
-    for(auto &arg : args)
+    for(auto const& arg : args)
       if(arg.type == "new_id")
         {
-          if(arg.interface == "")
+          if(arg.interface.empty())
             ss << "proxy_t &interface, uint32_t version, ";
         }
       else
@@ -270,20 +266,20 @@ struct request_t : public event_t
     return ss.str();
   }
 
-  std::string print_body(std::string interface_name)
+  std::string print_body(const std::string& interface_name) const
   {
     std::stringstream ss;
-    if(ret.name == "")
+    if(ret.name.empty())
       ss <<  "void ";
     else
       ss << ret.print_type() << " ";
     ss << interface_name << "_t::" << sanitise(name) << "(";
 
     bool new_id_arg = false;
-    for(auto &arg : args)
+    for(auto const& arg : args)
       if(arg.type == "new_id")
         {
-          if(arg.interface == "")
+          if(arg.interface.empty())
             {
               ss << "proxy_t &interface, uint32_t version, ";
               new_id_arg = true;
@@ -297,22 +293,22 @@ struct request_t : public event_t
     ss.seekp(0, std::ios_base::end);
     ss << ")\n{" << std::endl;
 
-    if(ret.name == "")
-      ss <<  "  marshal(" << opcode << "u, ";
-    else if(ret.interface == "")
+    if(ret.name.empty())
+      ss <<  "  marshal(" << opcode << "U, ";
+    else if(ret.interface.empty())
       {
-        ss << "  proxy_t p = marshal_constructor_versioned(" << opcode << "u, interface.interface, version, ";
+        ss << "  proxy_t p = marshal_constructor_versioned(" << opcode << "U, interface.interface, version, ";
       }
     else
       {
-        ss << "  proxy_t p = marshal_constructor(" << opcode << "u, &" << ret.interface << "_interface, ";
+        ss << "  proxy_t p = marshal_constructor(" << opcode << "U, &" << ret.interface << "_interface, ";
       }
 
-    for(auto &arg : args)
+    for(auto const& arg : args)
       {
         if(arg.type == "new_id")
           {
-            if(arg.interface == "")
+            if(arg.interface.empty())
               ss << "std::string(interface.interface->name), version, ";
             ss << "nullptr, ";
           }
@@ -320,7 +316,7 @@ struct request_t : public event_t
           ss << "argument_t::fd(" << sanitise(arg.name) << "), ";
         else if(arg.type == "object")
           ss << sanitise(arg.name) << ".proxy_has_object() ? reinterpret_cast<wl_object*>(" << sanitise(arg.name) << ".c_ptr()) : nullptr, ";
-        else if(arg.enum_name != "")
+        else if(!arg.enum_name.empty())
           ss << "static_cast<" << arg.print_enum_wire_type() << ">(" << sanitise(arg.name) + "), ";
         else
           ss << sanitise(arg.name) + ", ";
@@ -330,7 +326,7 @@ struct request_t : public event_t
     ss.seekp(0, std::ios_base::end);
     ss << ");" << std::endl;
 
-    if(ret.name != "")
+    if(!ret.name.empty())
       {
         if(new_id_arg)
           {
@@ -363,11 +359,11 @@ struct enum_entry_t : public element_t
 struct enumeration_t : public element_t
 {
   std::list<enum_entry_t> entries;
-  bool bitfield;
-  int id;
-  uint32_t width;
+  bool bitfield = false;
+  int id = 0;
+  uint32_t width = 0;
 
-  std::string print_forward(std::string iface_name)
+  std::string print_forward(const std::string& iface_name) const
   {
     std::stringstream ss;
     if(!bitfield)
@@ -377,7 +373,7 @@ struct enumeration_t : public element_t
     return ss.str();
   }
 
-  std::string print_header(std::string iface_name)
+  std::string print_header(const std::string& iface_name) const
   {
     std::stringstream ss;
     ss << "/** \\brief " << summary << std::endl
@@ -395,9 +391,9 @@ struct enumeration_t : public element_t
          << "  " << iface_name << "_" << name << "(const uint32_t value)" << std::endl
          << "    : detail::bitfield<" << width << ", " << id << ">(value) {}" << std::endl;
 
-    for(auto &entry : entries)
+    for(auto const& entry : entries)
       {
-        if(entry.summary != "")
+        if(!entry.summary.empty())
           ss << "  /** \\brief " << entry.summary << " */" << std::endl;
 
         if(!bitfield)
@@ -417,11 +413,11 @@ struct enumeration_t : public element_t
     return ss.str();
   }
 
-  std::string print_body(std::string iface_name)
+  std::string print_body(const std::string& iface_name) const
   {
     std::stringstream ss;
     if(bitfield)
-      for(auto &entry : entries)
+      for(auto const& entry : entries)
         {
           ss << "const bitfield<" << width << ", " << id << "> " << iface_name << "_" << name
              << "::" << sanitise(entry.name) << "{" << entry.value << "};" << std::endl;
@@ -432,30 +428,30 @@ struct enumeration_t : public element_t
 
 struct interface_t : public element_t
 {
-  int version;
+  int version = 0;
   std::string orig_name;
-  int destroy_opcode;
+  int destroy_opcode = 0;
   std::list<request_t> requests;
   std::list<event_t> events;
   std::list<enumeration_t> enums;
 
-  std::string print_forward()
+  std::string print_forward() const
   {
     std::stringstream ss;
     ss << "class " << name << "_t;" << std::endl;
-    for(auto &e : enums)
+    for(auto const& e : enums)
       ss << e.print_forward(name);
     return ss.str();
   }
-  
-  std::string print_c_forward()
+
+  std::string print_c_forward() const
   {
     std::stringstream ss;
     ss << "struct " << orig_name << ";" << std::endl;
     return ss.str();
   }
 
-  std::string print_header()
+  std::string print_header() const
   {
     std::stringstream ss;
     ss << "/** \\brief " << summary << std::endl
@@ -468,14 +464,14 @@ struct interface_t : public element_t
        << "  struct events_t : public detail::events_base_t" << std::endl
        << "  {" << std::endl;
 
-    for(auto &event : events)
+    for(auto const& event : events)
       ss << event.print_functional() << std::endl;
 
     ss << "  };" << std::endl
        << std::endl
-       << "  static int dispatcher(uint32_t opcode, std::vector<detail::any> args, std::shared_ptr<detail::events_base_t> e);" << std::endl
+       << "  static int dispatcher(uint32_t opcode, const std::vector<detail::any>& args, const std::shared_ptr<detail::events_base_t>& e);" << std::endl
        << std::endl
-       << "  " << name << "_t(proxy_t const &wrapped_proxy, construct_proxy_wrapper_tag);" << std::endl
+       << "  " << name << "_t(proxy_t const &wrapped_proxy, construct_proxy_wrapper_tag /*unused*/);" << std::endl
        << std::endl;
 
     ss << "public:" << std::endl
@@ -490,43 +486,43 @@ struct interface_t : public element_t
        << "  operator " << orig_name << "*() const;" << std::endl
        << std::endl;
 
-    for(auto &request : requests)
+    for(auto const& request : requests)
       if(request.name != "destroy")
         ss << request.print_header() << std::endl;
 
-    for(auto &event : events)
+    for(auto const& event : events)
       ss << event.print_signal_header() << std::endl;
 
     ss << "};" << std::endl
        << std::endl;
-    
-    for(auto &enumeration : enums)
+
+    for(auto const& enumeration : enums)
       ss << enumeration.print_header(name) << std::endl;
 
     return ss.str();
   }
 
-  std::string print_interface_header()
+  std::string print_interface_header() const
   {
     std::stringstream ss;
     ss << "  extern const wl_interface " << name << "_interface;" << std::endl;
     return ss.str();
   }
 
-  std::string print_body()
+  std::string print_body() const
   {
     std::stringstream set_events;
     set_events << "  if(proxy_has_object() && get_wrapper_type() == wrapper_type::standard)" << std::endl
                << "    {" << std::endl
                << "      set_events(std::shared_ptr<detail::events_base_t>(new events_t), dispatcher);" << std::endl;
     if(destroy_opcode != -1)
-      set_events << "      set_destroy_opcode(" << destroy_opcode << "u);" << std::endl;
+      set_events << "      set_destroy_opcode(" << destroy_opcode << "U);" << std::endl;
     set_events << "    }" << std::endl;
-      
+
     std::stringstream set_interface;
-    set_interface << "  interface = &" << name << "_interface;" << std::endl
-                  << "  copy_constructor = [] (const proxy_t &p) -> proxy_t" << std::endl
-                  << "    { return " << name << "_t(p); };" << std::endl;
+    set_interface << "  set_interface(&" << name << "_interface);" << std::endl
+                  << "  set_copy_constructor([] (const proxy_t &p) -> proxy_t" << std::endl
+                  << "    { return " << name << "_t(p); });" << std::endl;
 
     std::stringstream ss;
     ss << name << "_t::" << name << "_t(const proxy_t &p)" << std::endl
@@ -548,7 +544,7 @@ struct interface_t : public element_t
        << set_interface.str()
        << "}" << std::endl
        << std::endl
-       << name << "_t::" << name << "_t(proxy_t const &wrapped_proxy, construct_proxy_wrapper_tag)" << std::endl
+       << name << "_t::" << name << "_t(proxy_t const &wrapped_proxy, construct_proxy_wrapper_tag /*unused*/)" << std::endl
        << "  : proxy_t(wrapped_proxy, construct_proxy_wrapper_tag())"
        << "{" << std::endl
        << set_interface.str()
@@ -567,77 +563,77 @@ struct interface_t : public element_t
        << "}" << std::endl
        << std::endl;
 
-    for(auto &request : requests)
+    for(auto const& request : requests)
       if(request.name != "destroy")
         ss << request.print_body(name) << std::endl
            << std::endl;
 
-    for(auto &event : events)
+    for(auto const& event : events)
       ss << event.print_signal_body(name) << std::endl;
 
-    ss << "int " << name << "_t::dispatcher(uint32_t opcode, std::vector<any> args, std::shared_ptr<detail::events_base_t> e)" << std::endl
+    ss << "int " << name << "_t::dispatcher(uint32_t opcode, const std::vector<any>& args, const std::shared_ptr<detail::events_base_t>& e)" << std::endl
        << "{" << std::endl;
 
-    if(events.size())
+    if(!events.empty())
       {
         ss << "  std::shared_ptr<events_t> events = std::static_pointer_cast<events_t>(e);" << std::endl
            << "  switch(opcode)" << std::endl
            << "    {" << std::endl;
-        
+
         int opcode = 0;
-        for(auto &event : events)
+        for(auto const& event : events)
           ss << event.print_dispatcher(opcode++) << std::endl;
-        
+
         ss << "    }" << std::endl;
       }
 
     ss << "  return 0;" << std::endl
        << "}" << std::endl;
 
-    for(auto &enumeration : enums)
+    for(auto const& enumeration : enums)
       ss << enumeration.print_body(name) << std::endl;
 
     return ss.str();
   }
 
-  std::string print_interface_body()
+  std::string print_interface_body() const
   {
     std::stringstream ss;
-    for(auto &request : requests)
+    for(auto const& request : requests)
       {
         ss << "const wl_interface* " << name << "_interface_" << request.name << "_request[" << request.args.size() << "] = {" << std::endl;
-        for(auto &arg : request.args)
-          if(arg.interface != "")
+        for(auto const& arg : request.args)
+          if(!arg.interface.empty())
             ss  << "  &" << arg.interface << "_interface," << std::endl;
           else
-            ss  << "  NULL," << std::endl;
+            ss  << "  nullptr," << std::endl;
         ss << "};" << std::endl
            << std::endl;
       }
-    for(auto &event : events)
+    for(auto const& event : events)
       {
         ss << "const wl_interface* " << name << "_interface_" << event.name << "_event[" << event.args.size() << "] = {" << std::endl;
-        for(auto &arg : event.args)
-          if(arg.interface != "")
+        for(auto const& arg : event.args)
+          if(!arg.interface.empty())
             ss  << "  &" << arg.interface << "_interface," << std::endl;
           else
-            ss  << "  NULL," << std::endl;
+            ss  << "  nullptr," << std::endl;
         ss << "};" << std::endl
            << std::endl;
       }
     ss << "const wl_message " << name << "_interface_requests[" << requests.size() << "] = {" << std::endl;
-    for(auto &request : requests)
+    for(auto const& request : requests)
       {
         ss << "  {" << std::endl
            << "    \"" << request.name << "\"," << std::endl
            << "    \"";
         if(request.since > 1)
           ss << request.since;
-        for(auto &arg : request.args)
+        for(auto const& arg : request.args)
           {
             if(arg.allow_null)
               ss << "?";
-            if(arg.type == "new_id" && arg.interface == "")
+            if(arg.type == "new_id" && arg.interface.empty())
               ss << "su";
             ss << arg.print_short();
           }
@@ -648,18 +644,18 @@ struct interface_t : public element_t
     ss << "};" << std::endl
        << std::endl;
     ss << "const wl_message " << name << "_interface_events[" << events.size() << "] = {" << std::endl;
-    for(auto &event : events)
+    for(auto const& event : events)
       {
         ss << "  {" << std::endl
            << "    \"" << event.name << "\"," << std::endl
            << "    \"";
         if(event.since > 1)
           ss << event.since;
-        for(auto &arg : event.args)
+        for(auto const& arg : event.args)
           {
             if(arg.allow_null)
               ss << "?";
-            if(arg.type == "new_id" && arg.interface == "")
+            if(arg.type == "new_id" && arg.interface.empty())
               ss << "su";
             ss << arg.print_short();
           }
@@ -686,10 +682,10 @@ struct interface_t : public element_t
 
 std::string unprefix(const std::string &name)
 {
-  std::string::size_type prefix_len = name.find("_");
+  auto prefix_len = name.find('_');
   if(prefix_len != std::string::npos)
     {
-      std::string prefix = name.substr(0, prefix_len);
+      auto prefix = name.substr(0, prefix_len);
       if(prefix == "wl" || prefix == "wp")
         return name.substr(prefix_len+1, name.size());
     }
@@ -742,9 +738,9 @@ int main(int argc, char *argv[])
     {
       xml_document doc;
       doc.load_file(extra[c].c_str());
-      xml_node protocol = doc.child("protocol");
+      auto protocol = doc.child("protocol");
 
-      for(xml_node &interface : protocol.children("interface"))
+      for(auto const& interface : protocol.children("interface"))
         {
           interface_t iface;
           iface.destroy_opcode = -1;
@@ -756,7 +752,7 @@ int main(int argc, char *argv[])
             iface.version = 1;
           if(interface.child("description"))
             {
-              xml_node description = interface.child("description");
+              auto description = interface.child("description");
               iface.summary = description.attribute("summary").value();
               iface.description = description.text().get();
             }
@@ -764,7 +760,7 @@ int main(int argc, char *argv[])
           interface_names.push_back(iface.name);
 
           int opcode = 0; // Opcodes are in order of the XML. (Sadly undocumented)
-          for(xml_node &request : interface.children("request"))
+          for(auto const& request : interface.children("request"))
             {
               request_t req;
               req.opcode = opcode++;
@@ -777,7 +773,7 @@ int main(int argc, char *argv[])
 
               if(request.child("description"))
                 {
-                  xml_node description = request.child("description");
+                  auto description = request.child("description");
                   req.summary = description.attribute("summary").value();
                   req.description = description.text().get();
                 }
@@ -785,7 +781,7 @@ int main(int argc, char *argv[])
               // destruction takes place through the class destuctor
               if(req.name == "destroy")
                 iface.destroy_opcode = req.opcode;
-              for(xml_node &argument : request.children("arg"))
+              for(auto const& argument : request.children("arg"))
                 {
                   argument_t arg;
                   arg.type = argument.attribute("type").value();
@@ -812,10 +808,7 @@ int main(int argc, char *argv[])
                         }
                     }
 
-                  if(argument.attribute("allow-null") && std::string(argument.attribute("allow-null").value()) == "true")
-                    arg.allow_null = true;
-                  else
-                    arg.allow_null = false;
+                  arg.allow_null = argument.attribute("allow-null") && std::string(argument.attribute("allow-null").value()) == "true";
 
                   if(arg.type == "new_id")
                     req.ret = arg;
@@ -824,7 +817,7 @@ int main(int argc, char *argv[])
               iface.requests.push_back(req);
             }
 
-          for(xml_node &event : interface.children("event"))
+          for(auto const& event : interface.children("event"))
             {
               event_t ev;
               ev.name = event.attribute("name").value();
@@ -836,12 +829,12 @@ int main(int argc, char *argv[])
 
               if(event.child("description"))
                 {
-                  xml_node description = event.child("description");
+                  auto description = event.child("description");
                   ev.summary = description.attribute("summary").value();
                   ev.description = description.text().get();
                 }
 
-              for(xml_node &argument : event.children("arg"))
+              for(auto const& argument : event.children("arg"))
                 {
                   argument_t arg;
                   arg.type = argument.attribute("type").value();
@@ -868,23 +861,20 @@ int main(int argc, char *argv[])
                         }
                     }
 
-                  if(argument.attribute("allow-null") && std::string(argument.attribute("allow-null").value()) == "true")
-                    arg.allow_null = true;
-                  else
-                    arg.allow_null = false;
+                  arg.allow_null = argument.attribute("allow-null") && std::string(argument.attribute("allow-null").value()) == "true";
 
                   ev.args.push_back(arg);
                 }
               iface.events.push_back(ev);
             }
 
-          for(xml_node &enumeration : interface.children("enum"))
+          for(auto const& enumeration : interface.children("enum"))
             {
               enumeration_t enu;
               enu.name = enumeration.attribute("name").value();
               if(enumeration.child("description"))
                 {
-                  xml_node description = enumeration.child("description");
+                  auto description = enumeration.child("description");
                   enu.summary = description.attribute("summary").value();
                   enu.description = description.text().get();
                 }
@@ -899,7 +889,7 @@ int main(int argc, char *argv[])
               enu.id = enum_id++;
               enu.width = 0;
 
-              for(xml_node entry = enumeration.child("entry"); entry;
+              for(auto entry = enumeration.child("entry"); entry;
                   entry = entry.next_sibling("entry"))
                 {
                   enum_entry_t enum_entry;
@@ -912,7 +902,7 @@ int main(int argc, char *argv[])
                   if(entry.attribute("summary"))
                     enum_entry.summary = entry.attribute("summary").value();
 
-                  uint32_t tmp = static_cast<uint32_t> (std::log2(stol(enum_entry.value, nullptr, 0))) + 1u;
+                  auto tmp = static_cast<uint32_t>(std::log2(stol(enum_entry.value, nullptr, 0))) + 1U;
                   if(tmp > enu.width)
                     enu.width = tmp;
 
@@ -920,12 +910,13 @@ int main(int argc, char *argv[])
                 }
               iface.enums.push_back(enu);
             }
-          
+
           interfaces.push_back(iface);
         }
     }
 
-  std::string hpp_file(extra[extra.size()-2]), cpp_file(extra[extra.size()-1]);
+  std::string hpp_file(extra[extra.size()-2]);
+  std::string cpp_file(extra[extra.size()-1]);
   std::fstream wayland_hpp(hpp_file, std::ios_base::out | std::ios_base::trunc);
   std::fstream wayland_cpp(cpp_file, std::ios_base::out | std::ios_base::trunc);
 
@@ -947,16 +938,16 @@ int main(int argc, char *argv[])
   wayland_hpp << std::endl;
 
   // C forward declarations
-  for(auto &iface : interfaces)
+  for(auto const& iface : interfaces)
     if(iface.name != "display")
       wayland_hpp << iface.print_c_forward();
   wayland_hpp << std::endl;
-  
+
   wayland_hpp << "namespace wayland" << std::endl
               << "{" << std::endl;
 
   // C++ forward declarations
-  for(auto &iface : interfaces)
+  for(auto const& iface : interfaces)
     if(iface.name != "display")
       wayland_hpp << iface.print_forward();
   wayland_hpp << std::endl;
@@ -964,13 +955,13 @@ int main(int argc, char *argv[])
   // interface headers
   wayland_hpp << "namespace detail" << std::endl
               << "{" << std::endl;
-  for(auto &iface : interfaces)
+  for(auto const& iface : interfaces)
     wayland_hpp << iface.print_interface_header();
   wayland_hpp  << "}" << std::endl
   << std::endl;
 
   // class declarations
-  for(auto &iface : interfaces)
+  for(auto const& iface : interfaces)
     if(iface.name != "display")
       wayland_hpp << iface.print_header() << std::endl;
   wayland_hpp << std::endl
@@ -984,13 +975,13 @@ int main(int argc, char *argv[])
               << "using namespace wayland;" << std::endl
               << "using namespace detail;" << std::endl
               << std::endl;
-  
+
   // interface bodys
-  for(auto &iface : interfaces)
+  for(auto const& iface : interfaces)
     wayland_cpp << iface.print_interface_body();
 
   // class member definitions
-  for(auto &iface : interfaces)
+  for(auto const& iface : interfaces)
     if(iface.name != "display")
       wayland_cpp << iface.print_body() << std::endl;
   wayland_cpp << std::endl;
@@ -998,7 +989,7 @@ int main(int argc, char *argv[])
   // clean up
   wayland_hpp.close();
   wayland_cpp.close();
-  
+
   return 0;
 }
 
